 
######################################################################################################################
# Modeling seasional influenza with considering multiple locations and age groups populations in Taiwan
# Vaccinations are given as two matrices: 
# 1. vacc.routine -- vaccine rates for routine coverage
# 2. vacc.sia  -- vaccine rates for sia coverage
#    columns are vis.rate, vacc1, vacc2
###################################################################################################################### 

setwd("C://Taiwan Flu Model")

w <-5 
for(b in 1:w){ 
###################################################################################################################### 
# Load age groups populations at 22 counties
###################################################################################################################### 
popsize <-read.csv("county_pops.csv",header=T)				
numLocations <- nrow(popsize)		# 22 counties
numAges <- ncol(popsize) 		# 6 age groups: 0~3,4~12,13~18,19~49, 50~64,65+


###################################################################################################################### 
# Load weekly vaccination arrival percentage by age groups 
###################################################################################################################### 

#vacc.percent <- read.csv("C://Taiwan Flu Model/Vaccination_rates_300W.csv",header=T)				
vacc.percent <- read.csv("C://Taiwan Flu Model/Vaccination_rates_600W_Current.csv",header=T)
#vacc.percent <- read.csv("C://Taiwan Flu Model/Vaccination_rates_600W_19-49.csv",header=T)
#vacc.percent <- read.csv("C://Taiwan Flu Model/Vaccination_rates_600W_Every.csv",header=T)



###################################################################################################################### 
# vaccine ordering quantity
###################################################################################################################### 
vacc.inventory.child <- 300000
vacc.inventory.adult <- 5700000
#vacc.inventory.child <- 0
#vacc.inventory.adult <- 0


##################################################################################################################### 
# Distances
##################################################################################################################### 


# County coordinates in order of data sheets, coord: col 1 is long, col 2 is lat : centroids of districts
coord <- as.matrix(read.csv("coordinates_taiwan.csv", header = T, sep = ",")) 

# Calculate distance in kilometers
gcdist<-function (x1, y1, x2, y2){
    if (!exists("atan2"))
    atan2 <- tan
    r <- 360/(2 * pi)
    lon1 <- x1/r
    lat1 <- y1/r
    lon2 <- x2/r
    lat2 <- y2/r
    dlon <- lon2 - lon1
    dlat <- lat2 - lat1
    a <- (sin(dlat/2))^2 + cos(lat1) * cos(lat2) * (sin(dlon/2))^2
    c <- 2 * atan2(sqrt(a), sqrt(1 - a))
    return(6370 * c)
}

#dm is the Euclidean dist in kilometers
dm <- matrix(NA, ncol=numLocations, nrow=numLocations)
for(i in 1:numLocations) {
	for(j in i:numLocations) {
      	dm[i,j] <- gcdist(as.double(coord[i, 2]),as.double(coord[i, 3]), as.double(coord[j, 2]), as.double(coord[j, 3]))
      	dm[j,i]<-dm[i,j]
     	}      
}


###################################################################################################################### 
# Transmission rates are vary between 2 to 4 depending on ages, distances and time(week)
###################################################################################################################### 
beta_min <- 1.01
beta_max <- 3.4

  
###################################################################################################################### 
# Set risks of infection for each age group populations(2013-2016)
###################################################################################################################### 
# risks of age (2013-2016) 
age_infection_percent <- c(1,1,1,1,1,1) 

# risks of age (2009-2010) 
age_infection_percent <- c(1,1,1,1,1,1) 

age_risk <- age_infection_percent / max(age_infection_percent) #Normalized age_risk by dividing the maximum


###################################################################################################################### 
# Set risks of infection at each week depending on historical ILIs 
###################################################################################################################### 
# risks of week (2013-2016) 
ILI_percent <- c(0.953011775388624,	0.979276648024616,	0.945783523002419,	0.958748486909395,	0.972287577473297,	1.0080551710652,	0.953996122546339,	0.984335519887046,	1.0460283166091,	1.02895487234308,	1.00412286689019,	0.998276588695015,	1.17166436918344,	1.0218790608568,	1.04158538980612,	1.01666336821509,	1.06702039616278,	1.0750932874501,	1.07750417555784,	1.06080545968792,	1.08947091584999,	1.05588950961458,	1.04240948074805,	1.20606772972349,	1.10789871790324,	1.17569476355413,	1.1658624371891,	1.08080753289015,	1.30694828492551,	1.24417752444103,	1.23841984493109,	1.2646115501849,	1.32573277320067,	1.23321051823682,	0.985411798756795,	1.40644602654105,	1.38538767519955,	1.18642034288595,	1.12541988152658,	1.15595739335741,	1.15753233370719,	1.21875264968292,	1.05996148612535,	1.13532947443051,	1.20740401514487,	1.15738585820287,	1.10930391594533,	1.19127406661825,	1.19262235555918,	1.20801694591141,	1.27274275472497,	1.333961660282)

# risks of week (2009-2010) 
#ILI_percent <- c(0, 0,	0,	0,	0,	0,	0,	0,	0,	0.018142434,	0.046671598,	0.096439138,	0.251510989,	0.373816568,	0.431963229,	0.354480135,	0.313778529,	0.250042265,	0.346914624,	0.414232883,	0.54396661,	0.650295858,	0.665891801,	0.692878276,	0.806434911,	1,	0.771587067,	0.557142857,	0.324545647,	0.300982671,	0.255230347,	0.251141167,	0.263355875,	0.197020287,	0.159234996,	0.179480135,	0.148087489,	0.111612426,	0.173668639,	0.202831784,	0.253497464,	0.27850803,	0.204860524,	0.192994505,	0.185397295,	0.21421175,	0.21771978,	0.231762468,	0.178465765,	0.15716399,	0.138905325,	0)

time_risk <- ILI_percent


###################################################################################################################### 
# Compute and update number of populations of each compartment
###################################################################################################################### 
# Inputs:
# 1. initial sizes
# 2. mixing matrix for patches: diagonal is 0
# 3. transmission
# Outputs:
# 1. 
# 2. 
###################################################################################################################### 
#Stratify population by time, locatin, and age
T <- 53 								# This will run the simulation for one year (52 weeks plus 1 week as an initial state)
N <- array(0,c(T,numLocations, numAges))      #Total population
S <- array(0,c(T,numLocations, numAges))      #Susceptible population
V <- array(0,c(T,numLocations, numAges))			#Vaccinated population
I <- array(0,c(T,numLocations, numAges))      #Infected population
R <- array(0,c(T,numLocations, numAges))      #Recovered population

N.t <- array(0,c(T))         #To output population at each time period
S.t <- array(0,c(T))         
V.t <- array(0,c(T))         
I.t <- array(0,c(T))
R.t <- array(0,c(T))         
New_I.t <- array(0,c(T))

kk.t <-array(0,c(T))       #These kk matrices are for R code verification only
kk2.17 <-array(0,c(numLocations))
kk3.17 <-array(0,c(numAges,numLocations))
kk4.17 <-array(0,c(numLocations))
kk5.17 <-array(0,c(numAges))

for(a in 1:numAges){
	N[1,,a] <- t(as.numeric(popsize[,a]))		#Total populations at each location of different age group
}

S[1,,] <- N[1,,]							#Initial populations are all susceptible
#I[1,1:numLocations,] <- rpois(1,400)							#Initial infections
I[1,1:numLocations,1] <- c(9176,6247,4891,6355,4356,6419,1057,1256,1298,2978,1173,1612,1197,1939,512,766,236,859,1004,624,307,28)	


#----------------Main Procedure--------------------


for(t in 2:T-1){   

	#1.  Vaccination arrivals
	#1.1 Poisson vaccinations 			
	vacc.arrivals <-array(0,c(numLocations, numAges))
	for(j in 1:numLocations){  	
		vacc.arrivals[j,] <- rpois(numAges,S[t,j,] * t(vacc.percent[t,2:7])) #Assume that vaccine recipient arrivals follow Poisson distribution with mean equal to susceptible population x the prior arrival rate
		vacc.arrivals[j,] <- pmin(vacc.arrivals[j,], S[t,j,]) #Ensure that random generated arrivals <= susceptible population
	}
	kk.t[t] <- sum(vacc.arrivals)
	
	#2.2 Fairshared vaccines if arrivals > vaccine availabilities	
	#2.2.1 child vaccine
	if (vacc.inventory.child > 0){ 
		fill.rate <- vacc.inventory.child / pmax(1,sum(vacc.arrivals[,1])) #Compute the vaccine filled rate
		fill.rate <- pmin(1, fill.rate) 
		
		V[t,,1] <- round(fill.rate * vacc.arrivals[,1],0) #Determine the vaccinated population
		vacc.inventory.child <- vacc.inventory.child - sum(V[t,,1]) #Update the vaccine ending inventory
	}
	else { V[t,,1] <- 0}

	#2.2.2 For poulations >= 4yrs
	if (vacc.inventory.adult > 0){ 
		fill.rate <- vacc.inventory.adult/pmax(1,sum(vacc.arrivals[,2:numAges]))
		fill.rate <- pmin(1, fill.rate)			
		V[t,,2:numAges] <- round(fill.rate * vacc.arrivals[,2:numAges],0)
		vacc.inventory.adult <- vacc.inventory.adult - sum(V[t,,2:numAges])
	}
	else {V[t,,2:numAges] <- 0}			

	#Determine S/I/R for each county
	for(j in 1:numLocations){  		
 			
		#2.3 Immunized population = Vaccination population x Vaccine efficacy 
		nSR<- V[t,j,] * 0.59

		#3. New infections
		#3.1 Distance risk
		distance_risk <- c(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)  #Assume that the transmission risk is the same for each location.However, this assumption can be relaxed by resetting the arracy c.

		#3.2 Beta
		# Matrix of transmission rates at time t for a age population 
		# 	Each row represents to a transmission risk factor of different age group populations		
		# 	Each column represents to a distance risk of different location mixings
		beta <- time_risk[t] * age_risk %*% t(distance_risk)	

		#3.3 Poisson new infections of each age groups
		I.ByCounty <- rowSums(I[t,,])
		
		#I.ByCounty <- I.ByCounty + rpois(1,30)
		
		nSI <- rpois(numAges,((beta %*% I.ByCounty) / sum(N[t,,])) * (S[t,j,]-nSR))  #+rpois(1,30)
		nSI <- pmin(S[t,j,]-nSR, nSI)

		# For R code verfication only
		if (t == 17) {
			if (j ==1) {
				kk3.17 <- beta		
				kk4.17 <- I.ByCounty
				kk5.17 <- (beta %*% I.ByCounty / sum(N[t,,]))
			}
			kk2.17[j] <- sum(nSI)
		}	

		#4. Recovered (in average 3 days) 
		nIR <- pmin( rpois(numAges, I[t,j,] * 7/3), I[t,j,])
		
		#5. Update new compartments
		N[t+1,j,] <- S[t,j,] + I[t,j,] + R[t,j,]
		S[t+1,j,] <- S[t,j,] - nSI - nSR
		I[t+1,j,] <- I[t,j,] + nSI - nIR
		R[t+1,j,] <- R[t,j,] + nIR + nSR

		New_I.t[t] <- New_I.t[t] + sum(nSI)
		u <-New_I.t
	}

	
	N.t[t] <- sum(N[t,,])
	S.t[t] <- sum(S[t,,])
	V.t[t] <- sum(V[t,,])
	I.t[t] <- sum(I[t,,])
	R.t[t] <- sum(R[t,,])

	
}


#multirun.I [b] <- New_I.t
#write.table(multirun.I [b,52], file = "Outout_Infectionsssss.csv", sep = ",", row.names = TRUE, col.names = TRUE)

if (b==1)
  write.table(New_I.t, file = "Outout_Infections1.csv", sep = ",", row.names = TRUE, col.names = TRUE)
if (b==2)
  write.table(New_I.t, file = "Outout_Infections2.csv", sep = ",", row.names = TRUE, col.names = TRUE)
if (b==3)
  write.table(New_I.t, file = "Outout_Infections3.csv", sep = ",", row.names = TRUE, col.names = TRUE)
if (b==4)
  write.table(New_I.t, file = "Outout_Infections4.csv", sep = ",", row.names = TRUE, col.names = TRUE)
if (b==5)
  write.table(New_I.t, file = "Outout_Infections5.csv", sep = ",", row.names = TRUE, col.names = TRUE)
}

out1 <- read.csv("Outout_Infections1.csv",header=T,sep=",")
out2 <- read.csv("Outout_Infections2.csv",header=T,sep=",")	
out3 <- read.csv("Outout_Infections3.csv",header=T,sep=",")	
out4 <- read.csv("Outout_Infections4.csv",header=T,sep=",")	
out5 <- read.csv("Outout_Infections5.csv",header=T,sep=",")

colnames(out1) <- ("NewI_1")
colnames(out2) <- ("NewI_2")
colnames(out3) <- ("NewI_3")
colnames(out4) <- ("NewI_4")
colnames(out5) <- ("NewI_5")


#week <-c(40,41,42,43,44,45,46,47,48,49,50,51,52,53,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39)
week <- 1:53

out.1 <- data.frame(week,out1)
out.2 <- data.frame(week,out2)
out.3 <- data.frame(week,out3)
out.4 <- data.frame(week,out4)
out.5 <- data.frame(week,out5)


output1 <- merge(x=out.1, y=out.2, by.x="week", by.y="week")
output2 <- merge(x=output1, y=out.3, by.x="week", by.y="week")
output3 <- merge(x=output2, y=out.4, by.x="week", by.y="week")
output4 <- merge(x=output3, y=out.5, by.x="week", by.y="week")


write.table(output4, file = "Final_Outout_Infections.csv", sep = ",", row.names = TRUE, col.names = TRUE)

